# Code used in lectures of
# ST414: Advanced topics in Statistics
# Asymptotic Statistics
###
# 27th November 2009
# Ioannis Kosmidis, http://go.warwick.ac.uk/kosmidis

##########################################
### Complete enumeration for the behaviour
### of modified log-odds estimators
##########################################
##########################################


### Calculates the bias, variance and
### mean-squared error of estimators
### of the form log((Y + a)/(n - Y + a))
### through complete enumeration,
### for a supplied value of n and a
### and a specified "true" probability p
calc.moments <- function(p, n, a = 1/2) {
  y <- 0:n
  beta0 <- log(p/(1-p))
  betatilde <- log((y + a)/(n - y + a))
  expect <- sum(dbinom(y, n, p)*betatilde)
  bias <- expect - beta0
  variance <- sum(dbinom(y, n, p)*(betatilde - expect)^2)
  out <- c(bias, variance, bias^2 + variance)
  names(out) <- c("bias", "variance", "mse")
  out
}

### Plots of the bias, variance and mse
### of a range ps of "true" probabilities p
### and for a range as of  constants a
make.plot <- function(n, as, ps) {
  where1 <- length(ps)
  if (!(0.5%in%as))
    as <- sort(c(as, 0.5))
  res <- as.list(rep(0, length(as)))
  for (i in 1:length(as)) {
    res[[i]] <- sapply(ps, calc.moments, n = n, a = as[i])
    colnames(res[[i]]) <- ps
  }
  biases <- lapply(res, function(x) x["bias",])
  variances <- lapply(res, function(x) x["variance",])
  mses <- lapply(res, function(x) x["mse",])
  names(biases) <- names(variances) <- names(mses) <- as
  ## Bias
  dev.new()
  plot(ps, biases[["0.5"]], type = "l", col = "blue", lwd = 2,
       ylim = c(min(unlist(biases)), max(unlist(biases))),
       xlab = "p", ylab = "Bias",
       main = paste("Bias for n =", n))
  text(x = ps[0.1*where1], y = biases[["0.5"]][0.1*where1],
       labels = paste("a =", 0.5), cex = 0.5)
  for (a in as) {
    if (a!=0.5) {
      temp.biases <- biases[[as.character(a)]]
      points(ps, temp.biases, type = "l", col = "darkgrey")
      text(x = ps[0.1*where1], y = temp.biases[0.1*where1],
           labels = paste("a =", a), cex = 0.5)
    }
  }
  abline(0, 0, lty =2)
  ## Variance
  dev.new()
  plot(ps, variances[["0.5"]], type = "l", col = "blue", lwd = 2,
       ylim = c(0, max(unlist(variances))),
       xlab = "p", ylab = "Variance",
       main = paste("Variance for n =", n))
  text(x = ps[0.3*where1], y = variances[["0.5"]][0.3*where1],
       labels = paste("a =", 0.5), cex = 0.5)
  for (a in as) {
    if (a!=0.5) {
      temp.variances <- variances[[as.character(a)]]
      points(ps, temp.variances, type = "l",
             col = "darkgrey")
      text(x = ps[0.3*where1], y = temp.variances[0.3*where1],
           labels = paste("a =", a), cex = 0.5)
    }
  }
  abline(0, 0, lty =2)
  ## MSE
  dev.new()
  plot(ps, mses[["0.5"]], type = "l", col = "blue", lwd = 2,
       ylim = c(0, max(unlist(mses))),
       xlab = "p", ylab = "Mean squared error",
       main = paste("Mean squared error for n =", n))
  text(x = ps[0.3*where1], y = mses[["0.5"]][0.3*where1],
       labels = paste("a =", 0.5), cex = 0.5)
  for (a in as) {
    if (a!=0.5) {
      temp.mses <- mses[[as.character(a)]]
      points(ps, temp.mses, type = "l",  col = "darkgrey")
      text(x = ps[0.3*where1], y = temp.mses[0.3*where1],
           labels = paste("a =", a), cex = 0.5)
    }
  }
  abline(0, 0, lty = 2)
}

## Sequences as and ps gor make.plot
as <- seq(0.1, 0.9, 0.2)
ps <- seq(0.01, 0.99, 0.005)
## Set value of n
n <- 10

make.plot(n, as, ps)
