# Code used in lectures of
# ST414: Advanced topics in Statistics
# Asymptotic Statistics
###
# 3rd December 2009
# Ioannis Kosmidis, http://go.warwick.ac.uk/kosmidis


## Fisher information, bias, log-likelihood and
## penalized loglikelihood of beta
info <- function(beta) {
  probs <- plogis(beta*x)
  sum(x^2*m*probs*(1 - probs))
}

bias1 <- function(beta) {
  probs <- plogis(beta*x)
  -sum(x^3*m*probs*(1 - probs)*(1 - 2*probs))/(2*info(beta)^2)
}

loglik <- function(beta, nstat) {
  probs <- plogis(beta*x)
  beta*nstat + sum(m*log(1-probs))
}

pen.loglik <- function(beta, nstat) {
  loglik(beta, nstat) + 0.5 * log(info(beta))
}



require(combinat)
x <- -2:2
nobs <- length(x)
## Number of trials on each value of x
m <- 2
#######################################

### Produce all possible datasets
all.data <- hcube(rep(m+1, length(x)), scale = 1, translation = -1)
### Natural statistics
nat.stat <- colSums(t(all.data)*x)
nstats <- sort(unique(nat.stat))

### Calculate estimates for each value of the natural statistic
beta.ml <- beta.bc <- beta.br <- beta.jc <- rep(0, length(nstats))
for (i in 1:length(nstats)) {
  beta.ml[i] <- optimize(loglik, interval = c(-20, 20),
                         maximum = TRUE, nstat = nstats[i])$maximum
  beta.bc[i] <- beta.ml[i] - bias1(beta.ml[i])
  beta.br[i] <- optimize(pen.loglik, interval = c(-20, 20),
                         maximum = TRUE, nstat = nstats[i])$maximum
}

### Calculate sampling probabilities for several values of the true
### parameter beta0
beta0s <- c(0.5, 1)
sampling.probs <- matrix(0, length(nstats), length(beta0s))
for (i in 1:length(beta0s)) {
  all.probs <- apply(all.data, 1, function(dat)
                     prod(dbinom(dat, m, plogis(beta0s[i]*x))))
  sampling.probs[,i] <- tapply(all.probs, nat.stat, sum)
}


res <- cbind(nstats, beta.ml, beta.bc, beta.br, sampling.probs)
res
