# Code used in lectures of
# ST414: Advanced topics in Statistics
# Asymptotic Statistics
###
# 7th December 2009
# Ioannis Kosmidis, http://go.warwick.ac.uk/kosmidis

set.seed(10)
truenu <- 1
truelambda <- 1
ssize <- 20
y <- rweibull(ssize, shape = truenu, scale = truelambda^(-1/truenu))

## the Weibull log-likelihood
weibull.loglik <- function(beta, dat) {
  lambda <- beta[1]
  nu <- beta[2]
  sum(dweibull(dat, shape = nu,
                scale = lambda^(-1/nu), log = TRUE))
}

## the score function
weibull.score <- function(beta, dat) {
  n <- length(dat)
  nu <- beta[2]
  lambda <- beta[1]
  -c(-sum(dat^nu) + n/lambda,
     n/nu + sum(log(dat)) - lambda*sum(log(dat)*dat^nu))
}

## the observed information
weibull.info <- function(beta, dat) {
  n <- length(dat)
  lambda <- beta[1]
  nu <- beta[2]
  jll <- n/lambda^2
  jln <- sum(dat^nu*log(dat))
  jnn <- n/nu^2 + lambda*sum(dat^nu*(log(dat))^2)
  matrix(c(jll, jln, jln, jnn), nrow = 2, ncol =2)
}

weibull.prof.nu <- function(nu, dat) {
  n <- length(dat)
  n*log(nu) - n*log(sum(dat^nu)) + nu*sum(log(y))
}

weibull.prof.lambda <- function(lambda, dat) {
  obj <- function(nu) {
    weibull.loglik(c(lambda, nu), dat)
  }
  res <- optimize(obj, interval = c(0, 15), maximum = TRUE)
  res
}

## Maximum likelihood estimates
nn <- length(y)
nuhat <- uniroot(f = function(nu1) nn/nu1 + sum(log(y)) -
                 nn*sum(y^nu1*log(y))/sum(y^nu1),
                 interval = c(0.001, 10), tol = 1e-8)$root
lambdahat <- nn/sum(y^nuhat)
mles <- c(lambdahat, nuhat)
names(mles) <- c("lambdahat","nuhat")
## You can check that this is the solution by
# weibull.score(mles, dat = y)
#
## Estimated asymptotic standard errors for the MLE
jhat <- weibull.info(mles, dat = y)
jhatInverse <- solve(jhat)
mle.std.errs <- sqrt(diag(jhatInverse))


## below we set a grid. might need to be changed
## for different seed ot different sample size (currently 20)
lambdas <- seq(0.01, 2, length = 200)
nus <- seq(0.51, 3, length = 200)
parvalues <- expand.grid(lambdas, nus)
##
## levels to be plotted, colors, line widths
conf.levels <- c(0.001, 0.1, 0.3, 0.5, 0.7, 0.9, 0.95, 0.99, 0.999)
if (!(0.95%in%conf.levels))
  conf.levels <- c(conf.levels, 0.95)
color1 <- rep("black", length(conf.levels))
lwds <- rep(1, length(conf.levels))
lwds[which(conf.levels == 0.95)] <- 3
llikvalues <- apply(parvalues, 1, weibull.loglik, dat = y)
llikvalues <- matrix(llikvalues,
                     nrow = length(lambdas),
                     ncol = length(nus))

###
# Contours of l(lambda, nu) with the traces of l_p(lambda)
# and l_p(nu)
contour(lambdas, nus, llikvalues,
        levels = round(max(llikvalues) -
          0.5*qchisq(conf.levels, 1), 1),
        labels = round(max(llikvalues) -
          0.5*qchisq(conf.levels, 1), 1),
        xlab = expression(lambda),
        ylab = expression(nu),
        col = color1, lwd = lwds)
points(x = mles[1], y = mles[2], col = "red", pch =16)
title(expression(paste("Contours of ", l(lambda, nu))))
lambda.nus <- sapply(nus, function(x) 1/mean(y^x))
points(x = lambda.nus, y = nus, type = "l", lwd = 2,
       lty = 2, col = "red")
nus.lambdas <- sapply(lambdas, function(x)
                      weibull.prof.lambda(x, y)$maximum)
points(x = lambdas, y = nus.lambdas, type = "l", lwd = 2,
       lty = 2, col = "blue")
legend("topright", legend = c(expression(paste("Trace of ", l[p](nu))),
                     expression(paste("Trace of ", l[p](lambda)))),
       col = c("red", "blue"), lwd = 2, lty = 2)


###
# l_p(lambda)
dev.new()
profloglik.lambdas <- sapply(lambdas, function(x)
                             weibull.prof.lambda(x, dat = y)$objective)
plot(lambdas, profloglik.lambdas, type = "l", col = "blue",
     xlab = expression(lambda),
     ylab = expression(l[p](lambda)))
abline(max(profloglik.lambdas) - 0.5*qchisq(0.95, 1), 0, lty = 2)
title(expression(l[p](lambda)))

###
# l_p(nu)
dev.new()
profloglik.nus <- sapply(nus, weibull.prof.nu, dat = y)
plot(nus, profloglik.nus, type = "l", col = "red",
     xlab = expression(nu),
     ylab = expression(l[p](nu)))
abline(max(profloglik.nus) - 0.5*qchisq(0.95, 1), 0, lty = 2)
title(expression(l[p](nu)))     

     
